@@ -1,4 +1,4 @@
 from .core import APIFairy  # noqa: F401
 from .decorators import authenticate, arguments, body, response, \
-    other_responses  # noqa: F401
+    other_responses, webhook  # noqa: F401
 from .fields import FileField  # noqa: F401
