@@ -19,6 +19,7 @@ try:
 except ImportError:  # pragma: no cover
     HTTPBasicAuth = None
     HTTPTokenAuth = None
+from packaging.version import Version
 from werkzeug.http import HTTP_STATUS_CODES
 
 from apifairy.decorators import _webhooks
@@ -46,6 +47,7 @@ class APIFairy:
         self.version = app.config.get('APIFAIRY_VERSION', 'No version')
         self.apispec_path = app.config.get('APIFAIRY_APISPEC_PATH',
                                            '/apispec.json')
+        self.apispec_version = app.config.get('APIFAIRY_APISPEC_VERSION', None)
         self.apispec_decorators = app.config.get(
             'APIFAIRY_APISPEC_DECORATORS', [])
         self.ui = app.config.get('APIFAIRY_UI', 'redoc')
@@ -147,10 +149,19 @@ class APIFairy:
             tags[name] = tag
         tag_list = [tags[name] for name in tag_names]
         ma_plugin = MarshmallowPlugin(schema_name_resolver=resolver)
+        apispec_version = self.apispec_version
+        if apispec_version is None:
+            apispec_version = '3.1.0' if _webhooks else '3.0.3'
+        version = Version(apispec_version)
+        if version < Version('3.0.3'):
+            raise RuntimeError("Must use at openapi version '3.0.3' or newer")
+        elif version < Version('3.1.0') and _webhooks:
+            raise RuntimeError("Must use at least openapi version '3.1.0' "
+                               'when using the @webhook decorator')
         spec = APISpec(
             title=self.title,
             version=self.version,
-            openapi_version='3.1.0' if _webhooks else '3.0.3',
+            openapi_version=apispec_version,
             plugins=[ma_plugin],
             info=info,
             servers=servers,
@@ -188,11 +199,11 @@ class APIFairy:
                     else:  # pragma: no cover
                         raise RuntimeError('Unknown authentication scheme')
                     if name in auth_names:
-                        v = 2
-                        new_name = f'{name}_{v}'
+                        apispec_version = 2
+                        new_name = f'{name}_{apispec_version}'
                         while new_name in auth_names:  # pragma: no cover
-                            v += 1
-                            new_name = f'{name}_{v}'
+                            apispec_version += 1
+                            new_name = f'{name}_{apispec_version}'
                         name = new_name
                     auth_names.append(name)
         security = {}
@@ -319,8 +330,8 @@ class APIFairy:
                 if view_func._spec.get('body'):
                     schema = view_func._spec.get('body')[0]
                     location = view_func._spec.get('body')[1]
-                    media_type = 'application/json'
-                    if location == 'form':
+                    media_type = view_func._spec.get('body')[2]
+                    if media_type is None and location == 'form':
                         has_file = False
                         for field in schema.dump_fields.values():
                             if isinstance(field, apifairy_fields.FileField):
@@ -328,6 +339,8 @@ class APIFairy:
                                 break
                         media_type = 'application/x-www-form-urlencoded' \
                             if not has_file else 'multipart/form-data'
+                    if media_type is None:
+                        media_type = 'application/json'
                     operation['requestBody'] = {
                         'content': {
                             media_type: {
