@@ -1,6 +1,5 @@
 from functools import wraps
 
-import flask
 from flask import current_app, Response
 from webargs.flaskparser import FlaskParser as BaseFlaskParser
 
@@ -8,6 +7,7 @@ from apifairy.exceptions import ValidationError
 
 
 class FlaskParser(BaseFlaskParser):
+    USE_ARGS_POSITIONAL = False
     DEFAULT_VALIDATION_STATUS = 400
 
     def load_form(self, req, schema):
@@ -27,12 +27,15 @@ _webhooks = {}
 
 
 def _ensure_sync(f):
-    if flask.__version__ < '2.' or hasattr(f, '_sync_ensured'):
+    if hasattr(f, '_sync_ensured'):
         return f
 
     @wraps(f)
     def wrapper(*args, **kwargs):
-        return current_app.ensure_sync(f)(*args, **kwargs)
+        if hasattr(current_app, 'ensure_sync'):
+            return current_app.ensure_sync(f)(*args, **kwargs)
+        else:  # pragma: no cover
+            return f(*args, **kwargs)
 
     wrapper._sync_ensured = True
     return wrapper
@@ -65,18 +68,34 @@ def arguments(schema, location='query', **kwargs):
         if not hasattr(f, '_spec') or f._spec.get('args') is None:
             _annotate(f, args=[])
         f._spec['args'].append((schema, location))
-        return use_args(schema, location=location, **kwargs)(f)
+        arg_name = f'{location}_{schema.__class__.__name__}_args'
+
+        @wraps(f)
+        def _f(*args, **kwargs):
+            location_args = kwargs.pop(arg_name, {})
+            return f(*args, location_args, **kwargs)
+
+        return use_args(schema, location=location, arg_name=arg_name,
+                        **kwargs)(_f)
     return decorator
 
 
-def body(schema, location='json', **kwargs):
+def body(schema, location='json', media_type=None, **kwargs):
     if isinstance(schema, type):  # pragma: no cover
         schema = schema()
 
     def decorator(f):
         f = _ensure_sync(f)
-        _annotate(f, body=(schema, location))
-        return use_args(schema, location=location, **kwargs)(f)
+        _annotate(f, body=(schema, location, media_type))
+        arg_name = f'{location}_{schema.__class__.__name__}_args'
+
+        @wraps(f)
+        def _f(*args, **kwargs):
+            location_args = kwargs.pop(arg_name, {})
+            return f(*args, location_args, **kwargs)
+
+        return use_args(schema, location=location, arg_name=arg_name,
+                        **kwargs)(_f)
     return decorator
 
 
